@@ -6,12 +6,13 @@ use strict;
 use warnings;
 use English qw< -no_match_vars >;
 use POSIX qw< strftime >;
-our $VERSION = '0.727';
+our $VERSION = '0.727001';
 
 use Log::Log4perl::Tiny qw< :easy :dead_if_first LOGLEVEL >;
 use Template::Perlish;
 
-use Data::Tubes::Util qw< normalize_args shorter_sub_names sprintffy >;
+use Data::Tubes::Util
+  qw< normalize_args read_file_maybe shorter_sub_names sprintffy >;
 use Data::Tubes::Plugin::Util qw< identify log_helper >;
 use Data::Tubes::Plugin::Plumbing;
 my %global_defaults = (input => 'rendered',);
@@ -54,15 +55,31 @@ sub _filenames_generator {
 sub dispatch_to_files {
    my %args = normalize_args(
       @_,
-      {
-         %global_defaults,
-         name    => 'write dispatcher',
-         binmode => ':encoding(UTF-8)'
-      }
+      [
+         {
+            %global_defaults,
+            name    => 'write dispatcher',
+            binmode => ':encoding(UTF-8)'
+         },
+         'filename'
+      ],
    );
    identify(\%args);
    my $name = delete $args{name};    # so that it can be overridden
 
+   if (defined(my $filename = delete $args{filename})) {
+      my $ref = ref $filename;
+      if (! $ref) {
+         $args{filename_template} //= $filename;
+      }
+      elsif ($ref eq 'CODE') {
+         $args{filename_factory} //= $filename;
+      }
+      else {
+         LOGDIE "argument filename has invalid type $ref";
+      }
+   }
+
    my $factory = delete $args{filename_factory};
    if (!defined($factory) && defined($args{filename_template})) {
       my $tp = Template::Perlish->new(%{$args{tp_opts} || {}});
@@ -84,11 +101,15 @@ sub dispatch_to_files {
 sub write_to_files {
    my %args = normalize_args(
       @_,
-      {
-         %global_defaults,
-         name    => 'write to file',
-         binmode => ':encoding(UTF-8)'
-      }
+      [
+         {
+            %global_defaults,
+            name     => 'write to file',
+            binmode  => ':encoding(UTF-8)',
+            filename => \*STDOUT,
+         },
+         'filename'
+      ],
    );
    identify(\%args);
    my $name = $args{name};
@@ -96,15 +117,18 @@ sub write_to_files {
    LOGDIE "$name: need an input"   unless defined $args{input};
 
    my $output = $args{filename};
-   $output = _filenames_generator($output)
-     unless ref($output);
+   $output = _filenames_generator($output) unless ref($output);
+
+   my %oha =
+     map { ($_ => $args{$_}) }
+     grep { defined $args{$_} } qw< binmode policy >;
+   for my $marker (qw< footer header interlude >) {
+      $oha{$marker} = read_file_maybe($args{$marker})
+        if defined $args{$marker};
+   }
    require Data::Tubes::Util::Output;
-   my $output_handler = Data::Tubes::Util::Output->new(
-      output => $output,
-      map { ($_ => $args{$_}) }
-        grep { exists $args{$_} }
-        qw< binmode footer header interlude policy >
-   );
+   my $output_handler =
+     Data::Tubes::Util::Output->new(%oha, output => $output,);
 
    my $input = $args{input};
    return sub {
